﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [StorableClass]
  [Item("VariableVectorAction", "")]
  public class VariableVectorAction : ItemDictionary<StringValue, IActionVariable>, IAction {

    public IOrderedEnumerable<StringValue> Order { get { return this.Keys.OrderBy(x => x.Value); } }

    public int VirtualLength { get { return this.Values.Sum(x => x.VirtualLength); } }

    [StorableConstructor]
    protected VariableVectorAction(bool deserializing) : base(deserializing) { }
    protected VariableVectorAction(VariableVectorAction original, Cloner cloner)
      : base(original, cloner) {
    }
    public VariableVectorAction() : base() { }
    public VariableVectorAction(int capacity) : base(capacity) { }
    public VariableVectorAction(IDictionary<StringValue, IActionVariable> dictionary) : base(dictionary) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new VariableVectorAction(this, cloner);
    }

    protected override void OnItemsAdded(IEnumerable<KeyValuePair<StringValue, IActionVariable>> items) {
      foreach (var item in items) {
        if (!(item.Value is StringVariable || item.Value is IntVariable)) {
          throw new ArgumentException("Value has to be of type 'StringVariable' or 'IntVariable'");
        }
      }
      base.OnItemsAdded(items);
    }

    public bool Match(IAction target) {
      var targetCast = target as VariableVectorAction;
      if (targetCast == null) { return false; }
      if (this.Order.SequenceEqual(targetCast.Order)) { return false; }
      foreach (var keyValuePair in targetCast) {
        if (!this[keyValuePair.Key].MatchVariable(keyValuePair.Value)) {
          return false;
        }
      }
      return true;
    }

    public void Add(IEnumerable<IActionVariable> action) {
      foreach (var variable in action) {
        this.Add(new StringValue(variable.VariableName), variable);
      }
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      bool first = true;
      foreach (var variable in this.Values) {
        if (!first) {
          first = false;
        } else {
          sb.Append(";");
        }
        sb.Append(variable.ToString());
      }
      return sb.ToString();
    }

    public void Randomize(IRandom random) {
      foreach (var variable in this.Values) {
        variable.Randomize(random);
      }
    }

    public bool Identical(VariableVectorCondition target) {
      if (this.Order.SequenceEqual(target.Order)) { return false; }
      foreach (var keyValuePair in target) {
        if (!this[keyValuePair.Key].Identical(keyValuePair.Value)) {
          return false;
        }
      }
      return true;
    }
  }
}
