﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [StorableClass]
  [Item("VariableVectorCondition", "")]
  public class VariableVectorCondition : ItemDictionary<StringValue, IVariable>, ICondition {

    public IOrderedEnumerable<StringValue> Order { get { return this.Keys.OrderBy(x => x.Value); } }

    public int VirtualLength { get { return this.Values.Sum(x => x.VirtualLength); } }

    [StorableConstructor]
    protected VariableVectorCondition(bool deserializing) : base(deserializing) { }
    protected VariableVectorCondition(VariableVectorCondition original, Cloner cloner)
      : base(original, cloner) {
    }
    public VariableVectorCondition() : base() { }
    public VariableVectorCondition(int capacity) : base(capacity) { }
    public VariableVectorCondition(IDictionary<StringValue, IVariable> dictionary) : base(dictionary) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new VariableVectorCondition(this, cloner);
    }

    public bool Match(IInput target) {
      var targetCast = target as VariableVectorInput;
      if (targetCast == null) { return false; }
      if (this.Order.SequenceEqual(targetCast.Order)) { return false; }
      foreach (var keyValuePair in targetCast) {
        if (!this[keyValuePair.Key].MatchInput(keyValuePair.Value.Value)) {
          return false;
        }
      }
      return true;
    }

    public void Add(IEnumerable<IVariable> condition) {
      foreach (var variable in condition) {
        this.Add(new StringValue(variable.VariableName), variable);
      }
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      bool first = true;
      foreach (var variable in this.Values) {
        if (!first) {
          first = false;
        } else {
          sb.Append(";");
        }
        sb.Append(variable.ToString());
      }
      return sb.ToString();
    }

    public void Randomize(IRandom random, double spreadPercentage) {
      foreach (var variable in this.Values) {
        if (variable is DoubleVariable) {
          ((DoubleVariable)variable).Randomize(random, spreadPercentage);
        } else {
          variable.Randomize(random);
        }
      }
    }

    public bool Identical(VariableVectorCondition target) {
      if (this.Order.SequenceEqual(target.Order)) { return false; }
      foreach (var keyValuePair in target) {
        if (!this[keyValuePair.Key].Identical(keyValuePair.Value)) {
          return false;
        }
      }
      return true;
    }
  }
}
