﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [Item("VariableVectorCreator", "Description missing")]
  [StorableClass]
  public abstract class VariableVectorCreator : SingleSuccessorOperator, IVariableVectorCreator, IStochasticOperator {

    #region Parameter Properties
    public IValueLookupParameter<PercentValue> SpreadPercentageParameter {
      get { return (IValueLookupParameter<PercentValue>)Parameters["SpreadPercentage"]; }
    }
    public IValueLookupParameter<VariableVector> SampleVariableVectorParameter {
      get { return (IValueLookupParameter<VariableVector>)Parameters["SampleVariableVector"]; }
    }
    public ILookupParameter<VariableVector> VariableVectorParameter {
      get { return (ILookupParameter<VariableVector>)Parameters["VariableVector"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    #endregion

    [StorableConstructor]
    protected VariableVectorCreator(bool deserializing) : base(deserializing) { }
    protected VariableVectorCreator(VariableVectorCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public VariableVectorCreator()
      : base() {
      Parameters.Add(new ValueLookupParameter<PercentValue>("SpreadPercentage", ""));
      Parameters.Add(new ValueLookupParameter<VariableVector>("SampleVariableVector", ""));
      Parameters.Add(new LookupParameter<VariableVector>("VariableVector", ""));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
    }

    public override IOperation Apply() {
      VariableVectorParameter.ActualValue = Create(RandomParameter.ActualValue, SampleVariableVectorParameter.ActualValue, SpreadPercentageParameter.ActualValue.Value);
      return base.Apply();
    }

    protected abstract VariableVector Create(IRandom random, VariableVector sampleVariableVector, double spreadPercentage);
  }
}
